import { r as registerInstance, h, g as getElement } from './p-597600b0.js';
import { g as getControlSection, l as loadAndUpdatePageConfig, u as updateCurrentTabWithMappingsFromOtherTabs, c as updateResultants, d as getOutputSubCategoriesObj, e as updateSelectedOutputSubCategoryId, f as getOutputSection, h as getTabNamesObj, i as updateSelectedTabId, j as getElementKey, k as updateConfigData } from './p-f53384b7.js';
import './p-3ab399ac.js';

const adsCodeConversionCss = ":root{font-family:Roboto}.d-none{display:none}.v-hidden{visibility:hidden}.c-pointer{cursor:pointer}.disabled{cursor:not-allowed;opacity:0.6}.fw-bold{font-weight:bold}::-webkit-scrollbar-track{background-color:var(--color-ti-bg-grey);border-radius:5px}::-webkit-scrollbar{background-color:var(--color-ti-bg-grey);height:5px;width:5px}::-webkit-scrollbar-thumb{background-color:var(--color-ti-text-grey);border-radius:5px}ads-popup::part(popup-container){height:75%;width:75%}.page-header{box-sizing:content-box;display:flex;height:28px;justify-content:space-between;padding-bottom:5px}.page-header .page-title{color:var(--color-ti-text-grey);font-size:24px}.page-content{display:flex;height:calc(100% - 33px)}.page-content .left-section,.page-content .right-section{display:flex;flex-direction:column;gap:2%;height:100%;min-height:670px;padding:2%}.page-content .left-section{background-color:var(--color-ti-primary-white);min-width:540px;width:40%}.page-content .right-section{background-color:#f2f8fa;min-width:720px;width:60%}.popup-btn{align-items:center;cursor:pointer;display:flex;text-decoration:underline}.popup-content{display:flex;flex-direction:column}.popup-content img{margin:auto;max-height:calc(100% - 100px);max-width:calc(100% - 100px)}.popup-content .title{background-color:var(--color-ti-indicator-grey);font-size:1.5rem;font-weight:bold;height:40px;margin:0 auto;padding:5px;text-align:center;width:50%}.section-title{color:var(--color-ti-text-blue);font-size:18px}.section-tabs,.section-outputs{display:flex;flex-direction:column;height:100%;overflow:auto;width:100%}.section-outputs ads-tabs{--selected-tab-color:var(--color-ti-primary-blue-dark)}.section-outputs .tab-content{display:flex;flex-direction:column}.tab-content{height:max-content;overflow:hidden auto;width:100%}.tab-content ads-control{--control-label-width:50%}.tab-content .group-title{color:var(--color-ti-primary-red);font-size:1.2em;margin-top:5%}*{box-sizing:border-box}:host{background-color:var(--color-ti-bg-grey-dark);display:flex;flex-direction:column;height:100%;overflow:auto;padding:2%;width:100%}";

const AdsCodeConversion = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    // ---------- Internal variables ----------
    this.computationAPIs = {};
    this.isComputedValuesInitialized = false;
    // ----------- Helper methods -------------
    /**
     * To get control element
     * @param controlInfo
     * @param dataPath
     * @returns control element
     */
    this.getControlElement = (controlInfo, dataPath) => {
      return (h("ads-control", { key: getElementKey(controlInfo), controlInfo: controlInfo, dataPath: dataPath, onSetPathValue: ev => updateConfigData.call(this, ev) }));
    };
    /**
     * To get control groups
     * @returns Control groups
     */
    this.getControlSectionGroups = () => {
      const groupsInfo = getControlSection.call(this);
      let groups = [];
      groupsInfo.forEach(group => {
        groups.push(h("div", null, group.title ? h("div", { class: "group-title" }, group.title) : null, ...group.controls));
      });
      return groups;
    };
    /**
     * To get output info
     * @param outputInfo Output info object
     * @returns Output element
     */
    this.getOutputElement = (outputInfo, dataPath) => {
      return (h("ads-output", { "selected-device": this.selectedDevice, "tool-name": "code-conversion", key: getElementKey(outputInfo), outputInfo: outputInfo, dataPath: dataPath, onSetPathValue: ev => updateConfigData.call(this, ev) }));
    };
    this.selectedDevice = undefined;
    this.selectedPageIndex = undefined;
    this.selectedToolName = undefined;
    this.config = undefined;
    this.selectedTabId = undefined;
    this.selectedOutputSubCategoryId = undefined;
  }
  // ---------- Life cycle methods ----------
  componentDidLoad() {
    if (this.config === undefined && this.selectedDevice !== undefined) {
      this.loadPageConfig();
    }
  }
  // ----------- Watch handlers -------------
  loadPageConfig() {
    loadAndUpdatePageConfig.call(this, this.selectedToolName, this.selectedDevice);
  }
  onSelectedTabChanged() {
    if (this.config != undefined) {
      updateCurrentTabWithMappingsFromOtherTabs.call(this);
      updateResultants.call(this, true);
    }
  }
  render() {
    return this.config != undefined && this.isComputedValuesInitialized
      ? [
        h("div", { class: "page-header" }, h("div", { class: "page-title" }, "Code Conversion Tool"), h("ads-dropdown-menu", { name: "Related links", items: this.config.collaterals })),
        h("div", { class: "page-content" }, h("div", { class: "left-section", id: "left-section" }, h("div", { class: "section-title" }, this.config.label), h("div", { class: "section-tabs" }, h("ads-tabs", { tabNamesObj: getTabNamesObj(this.config.tabs), selectedTabId: this.selectedTabId, selectedPageIndex: this.selectedPageIndex, onUpdateSelectedTab: ev => updateSelectedTabId.call(this, ev), "hide-if-single-tab": true }), h("div", { class: "tab-content" }, this.getControlSectionGroups()))), h("div", { class: "right-section", id: "right-section" }, h("div", { class: "section-title" }, "Resultant Calculations"), h("div", { class: "section-outputs" }, this.selectedTabId != undefined && (h("ads-tabs", { tabNamesObj: getOutputSubCategoriesObj.call(this), selectedTabId: this.selectedOutputSubCategoryId, selectedPageIndex: this.selectedPageIndex, onUpdateSelectedTab: ev => updateSelectedOutputSubCategoryId.call(this, ev), "hide-if-single-tab": true })), h("div", { class: "tab-content" }, getOutputSection.call(this))))),
      ]
      : [];
  }
  get host() { return getElement(this); }
  static get watchers() { return {
    "selectedDevice": ["loadPageConfig"],
    "selectedTabId": ["onSelectedTabChanged"]
  }; }
};
AdsCodeConversion.style = adsCodeConversionCss;

export { AdsCodeConversion as ads_code_conversion };

//# sourceMappingURL=p-c520da23.entry.js.map