import * as utility from "./toolUtil.js";
let tue = {};
let isTueCalculated = false;
// ---------- Controls -----------
export function updateInputVoltage(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  let adc_fsr =
    args.gain_int.options !== undefined
      ? parseFloat(
          (
            (parseFloat(configData.adcScalingFactor) * args.vref.value) /
            (args.gain_int.options[args.gain_int.value] * args.gain_ext.value)
          ).toFixed(5),
        )
      : (args.v_in.value + 0.1) * 2;
  return {
    value:
      args.v_in.value < adc_fsr / 2
        ? args.v_in.value
        : (parseFloat(adc_fsr / 2) - 0.00001).toFixed(5),
    maxValue: (parseFloat(adc_fsr / 2) - 0.00001).toFixed(5),
  };
}

export function updateVrefVoltage(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  return {
    value:
      args.vref.value !== undefined
        ? args.vref.value
        : configData.vrefVoltage &&
          configData.vrefVoltage != "" &&
          typeof configData.vrefVoltage === "string"
        ? parseFloat(configData.vrefVoltage)
        : NaN,
    minValue: parseFloat(configData.minimumAdcVrefRange),
    maxValue: parseFloat(configData.maximumAdcVrefRange),
  };
}

export function initializeInternalGain(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.internalGain) {
    let internalGainObject = get_Control_Object(configData.internalGain);
    return {
      options: internalGainObject.options,
      value:
        args.gain_int.value !== undefined
          ? args.gain_int.value
          : internalGainObject.value,
    };
  }
  return {};
}

export function updateTotalGain(args) {
  let internalGain =
    args.gain_int.value !== undefined
      ? args.gain_int.options[args.gain_int.value] === "Bypassed"
        ? 1
        : args.gain_int.options[args.gain_int.value]
      : 1;
  return {
    value: internalGain * args.gain_ext.value,
  };
}

export function updateFsr(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  return {
    value:
      (parseFloat(configData.adcScalingFactor) * args.vref.value) /
      args.gain_total.value,
  };
}

export function updateFsrUtilization(args) {
  return {
    value: (args.v_in.value / args.adc_fsr.value) * 100,
  };
}

export function updateMinimumAmbientTemperature(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  return {
    isHidden: !args.temp_error.value,
    value:
      args.sys_temp_min.value !== undefined
        ? args.sys_temp_min.value
        : configData.minimumAmbientTemperature &&
          configData.minimumAmbientTemperature != "" &&
          typeof configData.minimumAmbientTemperature === "string"
        ? parseFloat(configData.minimumAmbientTemperature)
        : NaN,
    minValue: parseFloat(configData.minimumAdcTempRange),
    maxValue: args.sys_temp_max.value - 1,
  };
}

export function updateMaximumAmbientTemperature(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  return {
    isHidden: !args.temp_error.value,
    value:
      args.sys_temp_max.value !== undefined
        ? args.sys_temp_max.value
        : configData.maximumAmbientTemperature &&
          configData.maximumAmbientTemperature != "" &&
          typeof configData.maximumAmbientTemperature === "string"
        ? parseFloat(configData.maximumAmbientTemperature)
        : NaN,
    minValue: args.sys_temp_min.value + 1,
    maxValue: parseFloat(configData.maximumAdcTempRange),
  };
}

export function updateCalibrationType(args) {
  let calibrationTypes = args.cal_type.options;
  let currentCalibrationType = args.cal_type.value;
  if (args.temp_error.value && calibrationTypes.length === 3) {
    calibrationTypes.push({
      display: "Offset, gain, and temp calibration",
      value: 3,
    });
  } else if (!args.temp_error.value && calibrationTypes.length === 4) {
    currentCalibrationType =
      currentCalibrationType === 3 ? 0 : currentCalibrationType;
    calibrationTypes.splice(-1);
  }
  return {
    options: calibrationTypes,
    value: currentCalibrationType,
  };
}

export function updateCalibrationSourceAccuracy(args) {
  return {
    isHidden:
      args.cal_type.options[args.cal_type.value].display === "No calibration" ||
      args.cal_type.options[args.cal_type.value].display ===
        "Offset calibration",
  };
}

export function updateTemperatureCalibrationStepSize(args) {
  return {
    isHidden:
      args.cal_type.options[args.cal_type.value].display !==
      "Offset, gain, and temp calibration",
    value:
      args.cal_step.isHidden === false &&
      args.cal_step.value !== undefined &&
      args.cal_step.value <
        Math.abs(args.sys_temp_max.value - args.sys_temp_min.value)
        ? args.cal_step.value
        : Math.abs(args.sys_temp_max.value - args.sys_temp_min.value),
    maxValue: Math.abs(args.sys_temp_max.value - args.sys_temp_min.value),
  };
}

export function initializeTypicalAdcNoise(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcNoise) {
    let adcNoiseObject = get_Control_Object(configData.adcNoise);
    return {
      tooltip: configData.adcNoise.tooltip ? configData.adcNoise.tooltip : "",
      value:
        args.typ_adc_noise.value !== undefined
          ? args.typ_adc_noise.value
          : adcNoiseObject.typ,
    };
  }
  return {};
}

export function initializeMaximumAdcNoise(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcNoise) {
    let adcNoiseObject = get_Control_Object(configData.adcNoise);
    return {
      value:
        args.max_adc_noise.value !== undefined
          ? args.max_adc_noise.value
          : adcNoiseObject.max,
      unit: {
        controlType: "input-select",
        value:
          args.max_adc_noise.unit !== undefined
            ? args.max_adc_noise.unit.value
            : adcNoiseObject.unit,
        options: adcNoiseObject.options,
        class: "group-control-unit",
        invalidTooltip: false,
      },
    };
  }
  return {};
}

export function initializeTypicalAdcOffset(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcOffset) {
    let adcOffsetObject = get_Control_Object(configData.adcOffset);
    return {
      tooltip: configData.adcOffset.tooltip ? configData.adcOffset.tooltip : "",
      value:
        args.typ_adc_offset.value !== undefined
          ? args.typ_adc_offset.value
          : adcOffsetObject.typ,
    };
  }
  return {};
}

export function initializeMaximumAdcOffset(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcOffset) {
    let adcOffsetObject = get_Control_Object(configData.adcOffset);
    return {
      value:
        args.max_adc_offset.value !== undefined
          ? args.max_adc_offset.value
          : adcOffsetObject.max,
      unit: {
        controlType: "input-select",
        value:
          args.max_adc_offset.unit !== undefined
            ? args.max_adc_offset.unit.value
            : adcOffsetObject.unit,
        options: adcOffsetObject.options,
        class: "group-control-unit",
        invalidTooltip: false,
      },
    };
  }
  return {};
}

export function initializeTypicalAdcGainError(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcGainError) {
    let adcGainErrorObject = get_Control_Object(configData.adcGainError);
    return {
      tooltip: configData.adcGainError.tooltip
        ? configData.adcGainError.tooltip
        : "",
      value:
        args.typ_adc_ge.value !== undefined
          ? args.typ_adc_ge.value
          : adcGainErrorObject.typ,
    };
  }
  return {};
}

export function initializeMaximumAdcGainError(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcGainError) {
    let adcGainErrorObject = get_Control_Object(configData.adcGainError);
    return {
      value:
        args.max_adc_ge.value !== undefined
          ? args.max_adc_ge.value
          : adcGainErrorObject.max,
      unit: {
        controlType: "input-select",
        value:
          args.max_adc_ge.unit !== undefined
            ? args.max_adc_ge.unit.value
            : adcGainErrorObject.unit,
        options: adcGainErrorObject.options,
        class: "group-control-unit",
        invalidTooltip: false,
      },
    };
  }
  return {};
}

export function initializeTypicalAdcInl(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcInl) {
    let adcInlObject = get_Control_Object(configData.adcInl);
    return {
      tooltip: configData.adcInl.tooltip ? configData.adcInl.tooltip : "",
      value:
        args.typ_adc_inl.value !== undefined
          ? args.typ_adc_inl.value
          : adcInlObject.typ,
    };
  }
  return {};
}

export function initializeMaximumAdcInl(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcInl) {
    let adcInlObject = get_Control_Object(configData.adcInl);
    return {
      value:
        args.max_adc_inl.value !== undefined
          ? args.max_adc_inl.value
          : adcInlObject.max,
      unit: {
        controlType: "input-select",
        value:
          args.max_adc_inl.unit !== undefined
            ? args.max_adc_inl.unit.value
            : adcInlObject.unit,
        options: adcInlObject.options,
        class: "group-control-unit",
        invalidTooltip: false,
      },
    };
  }
  return {};
}

export function updateTypicalAdcOffsetDrift(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcOffsetDrift) {
    let adcOffsetDriftObject = get_Control_Object(configData.adcOffsetDrift);
    return {
      tooltip: configData.adcOffsetDrift.tooltip
        ? configData.adcOffsetDrift.tooltip
        : "",
      value:
        args.typ_adc_offset_drift.value !== undefined
          ? args.typ_adc_offset_drift.value
          : adcOffsetDriftObject.typ,
      isHidden: !args.temp_error.value,
    };
  }
  return {};
}

export function updateMaximumAdcOffsetDrift(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcOffsetDrift) {
    let adcOffsetDriftObject = get_Control_Object(configData.adcOffsetDrift);
    return {
      value:
        args.max_adc_offset_drift.value !== undefined
          ? args.max_adc_offset_drift.value
          : adcOffsetDriftObject.max,
      isHidden: !args.temp_error.value,
      unit: {
        controlType: "input-select",
        value:
          args.max_adc_offset_drift.unit !== undefined
            ? args.max_adc_offset_drift.unit.value
            : adcOffsetDriftObject.unit,
        options: adcOffsetDriftObject.options,
        class: "group-control-unit",
        invalidTooltip: false,
      },
    };
  }
  return {};
}

export function updateTypicalAdcGainDrift(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcGainDrift) {
    let adcGainDriftObject = get_Control_Object(configData.adcGainDrift);
    return {
      tooltip: configData.adcGainDrift.tooltip
        ? configData.adcGainDrift.tooltip
        : "",
      value:
        args.typ_adc_ge_drift.value !== undefined
          ? args.typ_adc_ge_drift.value
          : adcGainDriftObject.typ,
      isHidden: !args.temp_error.value,
    };
  }
  return {};
}

export function updateMaximumAdcGainDrift(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.adcGainDrift) {
    let adcGainDriftObject = get_Control_Object(configData.adcGainDrift);
    return {
      value:
        args.max_adc_ge_drift.value !== undefined
          ? args.max_adc_ge_drift.value
          : adcGainDriftObject.max,
      isHidden: !args.temp_error.value,
      unit: {
        controlType: "input-select",
        value:
          args.max_adc_ge_drift.unit !== undefined
            ? args.max_adc_ge_drift.unit.value
            : adcGainDriftObject.unit,
        options: adcGainDriftObject.options,
        class: "group-control-unit",
        invalidTooltip: false,
      },
    };
  }
  return {};
}

export function updateTypicalAmplifierOffsetDrift(args) {
  return {
    value: args.typ_amp_offset_drift.value,
    isHidden: !args.temp_error.value,
  };
}

export function updateMaximumAmplifierOffsetDrift(args) {
  let currentUnit = args.max_amp_offset_drift.unit;
  return {
    value: args.max_amp_offset_drift.value,
    isHidden: !args.temp_error.value,
    unit: currentUnit,
  };
}

export function updateTypicalAmplifierGainDrift(args) {
  return {
    value: args.typ_amp_ge_drift.value,
    isHidden: !args.temp_error.value,
  };
}

export function updateMaximumAmplifierGainDrift(args) {
  let currentUnit = args.max_amp_ge_drift.unit;
  return {
    value: args.max_amp_ge_drift.value,
    isHidden: !args.temp_error.value,
    unit: currentUnit,
  };
}

export function initializeTypicalVrefNoise(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.vrefNoise) {
    let vrefNoiseObject = get_Control_Object(configData.vrefNoise);
    return {
      tooltip: configData.vrefNoise.tooltip ? configData.vrefNoise.tooltip : "",
      value:
        args.typ_vref_noise.value !== undefined
          ? args.typ_vref_noise.value
          : vrefNoiseObject.typ,
    };
  }
  return {};
}

export function initializeMaximumVrefNoise(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.vrefNoise) {
    let vrefNoiseObject = get_Control_Object(configData.vrefNoise);
    return {
      value:
        args.max_vref_noise.value !== undefined
          ? args.max_vref_noise.value
          : vrefNoiseObject.max,
      unit: {
        controlType: "input-select",
        value:
          args.max_vref_noise.unit !== undefined
            ? args.max_vref_noise.unit.value
            : vrefNoiseObject.unit,
        options: vrefNoiseObject.options,
        class: "group-control-unit",
        invalidTooltip: false,
      },
    };
  }
  return {};
}

export function initializeTypicalVrefInitialAccuracy(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.vrefInitialAccuracy) {
    let vrefInitialAccuracyObject = get_Control_Object(
      configData.vrefInitialAccuracy,
    );
    return {
      tooltip: configData.vrefInitialAccuracy.tooltip
        ? configData.vrefInitialAccuracy.tooltip
        : "",
      value:
        args.typ_vref_initial_accuracy.value !== undefined
          ? args.typ_vref_initial_accuracy.value
          : vrefInitialAccuracyObject.typ,
    };
  }
  return {};
}

export function initializeMaximumVrefInitialAccuracy(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.vrefInitialAccuracy) {
    let vrefInitialAccuracyObject = get_Control_Object(
      configData.vrefInitialAccuracy,
    );
    return {
      value:
        args.max_vref_initial_accuracy.value !== undefined
          ? args.max_vref_initial_accuracy.value
          : vrefInitialAccuracyObject.max,
      unit: {
        controlType: "input-select",
        value:
          args.max_vref_initial_accuracy.unit !== undefined
            ? args.max_vref_initial_accuracy.unit.value
            : vrefInitialAccuracyObject.unit,
        options: vrefInitialAccuracyObject.options,
        class: "group-control-unit",
        invalidTooltip: false,
      },
    };
  }
  return {};
}

export function updateTypicalVrefTemperatureDrift(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.vrefTemperatureDrift) {
    let vrefTemperatureDriftObject = get_Control_Object(
      configData.vrefTemperatureDrift,
    );
    return {
      tooltip: configData.vrefTemperatureDrift.tooltip
        ? configData.vrefTemperatureDrift.tooltip
        : "",
      value:
        args.typ_vref_temperature_drift.value !== undefined
          ? args.typ_vref_temperature_drift.value
          : vrefTemperatureDriftObject.typ,
      isHidden: !args.temp_error.value,
    };
  }
  return {};
}

export function updateMaximumVrefTemperatureDrift(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.vrefTemperatureDrift) {
    let vrefTemperatureDriftObject = get_Control_Object(
      configData.vrefTemperatureDrift,
    );
    return {
      value:
        args.max_vref_temperature_drift.value !== undefined
          ? args.max_vref_temperature_drift.value
          : vrefTemperatureDriftObject.max,
      isHidden: !args.temp_error.value,
      unit: {
        controlType: "input-select",
        value:
          args.max_vref_temperature_drift.unit !== undefined
            ? args.max_vref_temperature_drift.unit.value
            : vrefTemperatureDriftObject.unit,
        options: vrefTemperatureDriftObject.options,
        class: "group-control-unit",
        invalidTooltip: false,
      },
    };
  }
  return {};
}

// ---------- Outputs ----------

export function initializeOutputUnit(args) {
  let configData = args.parameterConfigData[args.selectedDevice];
  if (configData.outputUnit) {
    let outputUnitObject = get_Control_Object(configData.outputUnit);
    return {
      options: outputUnitObject.options,
      value: outputUnitObject.unit,
    };
  }
  return {};
}

export function computeTableOfValues(args) {
  let table = [];
  let calibrationType = args.cal_type.options[args.cal_type.value].display;
  if (!isTueCalculated) {
    tue.adcNoise =
      args.type.value === "Typical"
        ? args.typ_adc_noise.value
        : args.max_adc_noise.value;
    tue.adcNoise = get_value_in_selected_unit(
      tue.adcNoise,
      args.max_adc_noise.unit.value,
      args.units.value,
      args,
    );
    tue.ampNoise =
      args.type.value === "Typical"
        ? args.typ_amp_noise.value
        : args.max_amp_noise.value;
    tue.ampNoise = get_value_in_selected_unit(
      tue.ampNoise,
      args.max_amp_noise.unit,
      args.units.value,
      args,
    );

    tue.vrefNoise =
      args.type.value === "Typical"
        ? args.typ_vref_noise.value
        : args.max_vref_noise.value;
    tue.vrefNoise = get_value_in_selected_unit(
      tue.vrefNoise,
      args.max_vref_noise.unit.value,
      args.units.value,
      args,
    );

    tue.adcLinearity =
      args.type.value === "Typical"
        ? args.typ_adc_inl
          ? args.typ_adc_inl.value
          : 0
        : args.max_adc_inl
        ? args.max_adc_inl.value
        : 0;
    tue.adcLinearity = get_value_in_selected_unit(
      tue.adcLinearity,
      args.max_adc_inl.unit.value,
      args.units.value,
      args,
    );

    tue.ampLinearity =
      args.type.value === "Typical"
        ? args.typ_amp_inl
          ? args.typ_amp_inl.value
          : 0
        : args.max_amp_inl
        ? args.max_amp_inl.value
        : 0;
    tue.ampLinearity = get_value_in_selected_unit(
      tue.ampLinearity,
      args.max_amp_inl.unit.value,
      args.units.value,
      args,
    );

    tue.vrefLinearity = "-";

    tue.adcGainError =
      args.type.value === "Typical"
        ? args.typ_adc_ge
          ? args.typ_adc_ge.value
          : 0
        : args.max_adc_ge
        ? args.max_adc_ge.value
        : 0;
    tue.adcGainError = get_value_in_selected_unit(
      tue.adcGainError,
      args.max_adc_ge.unit.value,
      args.units.value,
      args,
    );

    tue.ampGainError =
      args.type.value === "Typical"
        ? args.typ_amp_ge
          ? args.typ_amp_ge.value
          : 0
        : args.max_amp_ge
        ? args.max_amp_ge.value
        : 0;
    tue.ampGainError = get_value_in_selected_unit(
      tue.ampGainError,
      args.max_amp_ge.unit.value,
      args.units.value,
      args,
    );

    tue.vrefGainError =
      args.type.value === "Typical"
        ? args.typ_vref_initial_accuracy
          ? args.typ_vref_initial_accuracy.value
          : 0
        : args.max_vref_initial_accuracy
        ? args.max_vref_initial_accuracy.value
        : 0;
    tue.vrefGainError = get_value_in_selected_unit(
      tue.vrefGainError,
      args.max_vref_initial_accuracy.unit.value,
      args.units.value,
      args,
    );

    tue.adcOffset =
      args.type.value === "Typical"
        ? args.typ_adc_offset
          ? args.typ_adc_offset.value
          : 0
        : args.max_adc_offset
        ? args.max_adc_offset.value
        : 0;
    tue.adcOffset = get_value_in_selected_unit(
      tue.adcOffset,
      args.max_adc_offset.unit.value,
      args.units.value,
      args,
    );

    tue.ampOffset =
      args.type.value === "Typical"
        ? args.typ_amp_offset
          ? args.typ_amp_offset.value
          : 0
        : args.max_amp_offset
        ? args.max_amp_offset.value
        : 0;
    tue.ampOffset = get_value_in_selected_unit(
      tue.ampOffset,
      args.max_amp_offset.unit.value,
      args.units.value,
      args,
    );

    tue.vrefOffset = "-";

    tue.adcGainDrift =
      args.type.value === "Typical"
        ? args.typ_adc_ge_drift
          ? args.typ_adc_ge_drift.value
          : 0
        : args.max_adc_ge_drift
        ? args.max_adc_ge_drift.value
        : 0;
    tue.adcGainDrift = get_value_in_selected_unit(
      tue.adcGainDrift,
      args.max_adc_ge_drift.unit.value,
      args.units.value,
      args,
    );

    tue.ampGainDrift =
      args.type.value === "Typical"
        ? args.typ_amp_ge_drift
          ? args.typ_amp_ge_drift.value
          : 0
        : args.max_amp_ge_drift
        ? args.max_amp_ge_drift.value
        : 0;
    tue.ampGainDrift = get_value_in_selected_unit(
      tue.ampGainDrift,
      args.max_amp_ge_drift.unit.value,
      args.units.value,
      args,
    );

    tue.vrefGainDrift =
      args.type.value === "Typical"
        ? args.typ_vref_temperature_drift
          ? args.typ_vref_temperature_drift.value
          : 0
        : args.max_vref_temperature_drift
        ? args.max_vref_temperature_drift.value
        : 0;
    tue.vrefGainDrift = get_value_in_selected_unit(
      tue.vrefGainDrift,
      args.max_vref_temperature_drift.unit.value,
      args.units.value,
      args,
    );

    tue.adcOffsetDrift =
      args.type.value === "Typical"
        ? args.typ_adc_offset_drift
          ? args.typ_adc_offset_drift.value
          : 0
        : args.max_adc_offset_drift
        ? args.max_adc_offset_drift.value
        : 0;
    tue.adcOffsetDrift = get_value_in_selected_unit(
      tue.adcOffsetDrift,
      args.max_adc_offset_drift.unit.value,
      args.units.value,
      args,
    );

    tue.ampOffsetDrift =
      args.type.value === "Typical"
        ? args.typ_amp_offset_drift
          ? args.typ_amp_offset_drift.value
          : 0
        : args.max_amp_offset_drift
        ? args.max_amp_offset_drift.value
        : 0;
    tue.ampOffsetDrift = get_value_in_selected_unit(
      tue.ampOffsetDrift,
      args.max_amp_offset_drift.unit.value,
      args.units.value,
      args,
    );

    tue.vrefOffsetDrift = "-";

    tue.adcTotal = get_adc_Total(args);
    tue.ampTotal = get_amp_Total(args);
    tue.vrefTotal = get_vref_Total(args);

    tue.noiseTotal = get_noise_total(args);
    tue.offsetTotal = get_offset_total(args);
    tue.gainErrorTotal = get_gain_error_total(args);
    tue.inlTotal = get_inl_total(args);
    tue.calibratedOffsetDriftTotal = get_calibrated_offset_drift_total(args);
    tue.offsetDriftTotal = get_offset_drift_total(args);
    tue.calibratedGainDriftTotal = get_calibrated_gain_drift_total(args);
    tue.gainDriftTotal = get_gain_drift_total(args);
    tue.calibrationSourceAccuracy = get_value_in_selected_unit(
      args.cal_acc.value,
      args.cal_acc.unit,
      args.units.value,
      args,
    );
    calculate_and_update_tue_value(tue, calibrationType, args);
    isTueCalculated = true;
  } else {
    isTueCalculated = false;
  }
  let errorSourcesValueswithTempError = [
    { property: "adcOffsetDriftParameter", value: tue.adcOffsetDrift },
    { property: "ampOffsetDriftParameter", value: tue.ampOffsetDrift },
    { property: "vrefGainDriftParameter", value: tue.vrefGainDrift },
    { property: "adcGainDriftParameter", value: tue.adcGainDrift },
    { property: "ampGainDriftParameter", value: tue.ampGainDrift },
    { property: "adcOffsetParameter", value: tue.adcOffset },
    { property: "ampOffsetParameter", value: tue.ampOffset },
    { property: "adcGainErrorParameter", value: tue.adcGainError },
    { property: "ampGainErrorParameter", value: tue.ampGainError },
    { property: "vrefGainErrorParameter", value: tue.vrefGainError },
    { property: "adcLinearityParameter", value: tue.adcLinearity },
    { property: "ampLinearityParameter", value: tue.ampLinearity },
    { property: "ampNoiseParameter", value: tue.ampNoise },
    { property: "adcNoiseParameter", value: tue.adcNoise },
    { property: "vrefNoiseParameter", value: tue.vrefNoise },
  ];
  errorSourcesValueswithTempError.sort((a, b) => b.value - a.value);
  tue.highestErrorSourceWithTempError = [
    errorSourcesValueswithTempError[0],
    errorSourcesValueswithTempError[1],
    errorSourcesValueswithTempError[2],
  ];

  let errorSourcesValues = [
    { property: "adcOffsetParameter", value: tue.adcOffset },
    { property: "ampOffsetParameter", value: tue.ampOffset },
    { property: "adcGainErrorParameter", value: tue.adcGainError },
    { property: "ampGainErrorParameter", value: tue.ampGainError },
    { property: "vrefGainErrorParameter", value: tue.vrefGainError },
    { property: "adcLinearityParameter", value: tue.adcLinearity },
    { property: "ampLinearityParameter", value: tue.ampLinearity },
    { property: "ampNoiseParameter", value: tue.ampNoise },
    { property: "adcNoiseParameter", value: tue.adcNoise },
    { property: "vrefNoiseParameter", value: tue.vrefNoise },
  ];
  errorSourcesValues.sort((a, b) => b.value - a.value);
  tue.highestErrorSource = [
    errorSourcesValues[0],
    errorSourcesValues[1],
    errorSourcesValues[2],
  ];

  table.push(
    utility.getTableRow(
      [
        {
          value: "Parameters",
          colspan: 1,
          rowspan: 2,
        },
        {
          value: "Error Source",
          colspan: 3,
        },
        {
          value: "Total",
          colspan: calibrationType !== "No calibration" ? 3 : 2,
        },
      ],
      "head",
    ),
  );

  if (calibrationType !== "No calibration") {
    table.push(
      utility.getTableRow(
        [
          {
            value: "ADC",
          },
          {
            value: "AMP",
          },
          {
            value: "VREF",
          },
          {
            value: "Before Calibration",
          },
          {
            value: "After Calibration",
          },
        ],
        "head",
      ),
    );
    table.push(
      utility.getTableRow([
        {
          value: "Noise",
        },
        {
          value: parseFloat(tue.adcNoise.toPrecision(2)),
          classList: getClassName(tue, args, "adcNoiseParameter"),
        },
        {
          value: parseFloat(tue.ampNoise.toPrecision(2)),
          classList: getClassName(tue, args, "ampNoiseParameter"),
        },
        {
          value: parseFloat(tue.vrefNoise.toPrecision(2)),
          classList: getClassName(tue, args, "vrefNoiseParameter"),
        },
        {
          value: tue.noiseTotal,
        },
        {
          value: tue.noiseTotal,
        },
      ]),
    );
    table.push(
      utility.getTableRow([
        {
          value: "Offset",
        },
        {
          value: parseFloat(tue.adcOffset.toPrecision(2)),
          classList: getClassName(tue, args, "adcOffsetParameter"),
        },
        {
          value: parseFloat(tue.ampOffset.toPrecision(2)),
          classList: getClassName(tue, args, "ampOffsetParameter"),
        },
        {
          value: tue.vrefOffset,
        },
        {
          value: tue.offsetTotal,
        },
        {
          value: tue.noiseTotal,
        },
      ]),
    );
    table.push(
      utility.getTableRow([
        {
          value: "Gain Error",
        },
        {
          value: parseFloat(tue.adcGainError.toPrecision(2)),
          classList: getClassName(tue, args, "adcGainErrorParameter"),
        },
        {
          value: parseFloat(tue.ampGainError.toPrecision(2)),
          classList: getClassName(tue, args, "ampGainErrorParameter"),
        },
        {
          value: parseFloat(tue.vrefGainError.toPrecision(2)),
          classList: getClassName(tue, args, "vrefGainErrorParameter"),
        },
        {
          value: tue.gainErrorTotal,
        },
        {
          value:
            calibrationType !== "Offset calibration"
              ? tue.calibrationSourceAccuracy
              : tue.gainErrorTotal,
        },
      ]),
    );
    table.push(
      utility.getTableRow([
        {
          value: "INL",
        },
        {
          value: parseFloat(tue.adcLinearity.toPrecision(2)),
          classList: getClassName(tue, args, "adcLinearityParameter"),
        },
        {
          value: parseFloat(tue.ampLinearity.toPrecision(2)),
          classList: getClassName(tue, args, "ampLinearityParameter"),
        },
        {
          value: tue.vrefLinearity,
        },
        {
          value: tue.inlTotal,
        },
        {
          value: tue.inlTotal,
        },
      ]),
    );
    if (args.temp_error.value) {
      table.push(
        utility.getTableRow([
          {
            value: "Offset Drift",
          },
          {
            value: parseFloat(tue.adcOffsetDrift.toPrecision(2)),
            classList: getClassName(tue, args, "adcOffsetDriftParameter"),
          },
          {
            value: parseFloat(tue.ampOffsetDrift.toPrecision(2)),
            classList: getClassName(tue, args, "ampOffsetDriftParameter"),
          },
          {
            value: tue.vrefOffsetDrift,
          },
          {
            value: tue.offsetDriftTotal,
          },
          {
            value:
              args.temp_error.value &&
              calibrationType === "Offset, gain, and temp calibration"
                ? tue.calibratedOffsetDriftTotal
                : tue.offsetDriftTotal,
          },
        ]),
      );
      table.push(
        utility.getTableRow([
          {
            value: "Gain Drift",
          },
          {
            value: parseFloat(tue.adcGainDrift.toPrecision(2)),
            classList: getClassName(tue, args, "adcGainDriftParameter"),
          },
          {
            value: parseFloat(tue.ampGainDrift.toPrecision(2)),
            classList: getClassName(tue, args, "ampGainDriftParameter"),
          },
          {
            value: parseFloat(tue.vrefGainDrift.toPrecision(2)),
            classList: getClassName(tue, args, "vrefGainDriftParameter"),
          },
          {
            value: tue.gainDriftTotal,
          },
          {
            value:
              args.temp_error.value &&
              calibrationType === "Offset, gain, and temp calibration"
                ? tue.calibratedGainDriftTotal
                : tue.gainDriftTotal,
          },
        ]),
      );
    }
    table.push(
      utility.getTableRow([
        {
          value: "Total",
          classList: "table-total",
        },
        {
          value: tue.adcTotal,
          classList: "table-total",
        },
        {
          value: tue.ampTotal,
          classList: "table-total",
        },
        {
          value: tue.vrefTotal,
          classList: "table-total",
        },
        {
          value: tue.uncalibratedTue,
          classList: "table-total",
        },
        {
          value: tue.calibratedTue,
          classList: "table-total",
        },
      ]),
    );
  } else {
    table.push(
      utility.getTableRow(
        [
          {
            value: "ADC",
          },
          {
            value: "AMP",
          },
          {
            value: "VREF",
          },
          {
            value: "Before Calibration",
          },
        ],
        "head",
      ),
    );
    table.push(
      utility.getTableRow([
        {
          value: "Noise",
        },
        {
          value: parseFloat(tue.adcNoise.toPrecision(2)),
          classList: getClassName(tue, args, "adcNoiseParameter"),
        },
        {
          value: parseFloat(tue.ampNoise.toPrecision(2)),
          classList: getClassName(tue, args, "ampNoiseParameter"),
        },
        {
          value: parseFloat(tue.vrefNoise.toPrecision(2)),
          classList: getClassName(tue, args, "vrefNoiseParameter"),
        },
        {
          value: tue.noiseTotal,
        },
      ]),
    );
    table.push(
      utility.getTableRow([
        {
          value: "Offset",
        },
        {
          value: parseFloat(tue.adcOffset.toPrecision(2)),
          classList: getClassName(tue, args, "adcOffsetParameter"),
        },
        {
          value: parseFloat(tue.ampOffset.toPrecision(2)),
          classList: getClassName(tue, args, "ampOffsetParameter"),
        },
        {
          value: tue.vrefOffset,
        },
        {
          value: tue.offsetTotal,
        },
      ]),
    );
    table.push(
      utility.getTableRow([
        {
          value: "Gain Error",
        },
        {
          value: parseFloat(tue.adcGainError.toPrecision(2)),
          classList: getClassName(tue, args, "adcGainErrorParameter"),
        },
        {
          value: parseFloat(tue.ampGainError.toPrecision(2)),
          classList: getClassName(tue, args, "ampGainErrorParameter"),
        },
        {
          value: parseFloat(tue.vrefGainError.toPrecision(2)),
          classList: getClassName(tue, args, "vrefGainErrorParameter"),
        },
        {
          value: tue.gainErrorTotal,
        },
      ]),
    );
    table.push(
      utility.getTableRow([
        {
          value: "INL",
        },
        {
          value: parseFloat(tue.adcLinearity.toPrecision(2)),
          classList: getClassName(tue, args, "adcLinearityParameter"),
        },
        {
          value: parseFloat(tue.ampLinearity.toPrecision(2)),
          classList: getClassName(tue, args, "ampLinearityParameter"),
        },
        {
          value: tue.vrefLinearity,
        },
        {
          value: tue.inlTotal,
        },
      ]),
    );
    if (args.temp_error.value) {
      table.push(
        utility.getTableRow([
          {
            value: "Offset Drift",
          },
          {
            value: parseFloat(tue.adcOffsetDrift.toPrecision(2)),
            classList: getClassName(tue, args, "adcOffsetDriftParameter"),
          },
          {
            value: parseFloat(tue.ampOffsetDrift.toPrecision(2)),
            classList: getClassName(tue, args, "ampOffsetDriftParameter"),
          },
          {
            value: tue.vrefOffsetDrift,
          },
          {
            value: tue.offsetDriftTotal,
          },
        ]),
      );
      table.push(
        utility.getTableRow([
          {
            value: "Gain Drift",
          },
          {
            value: parseFloat(tue.adcGainDrift.toPrecision(2)),
            classList: getClassName(tue, args, "adcGainDriftParameter"),
          },
          {
            value: parseFloat(tue.ampGainDrift.toPrecision(2)),
            classList: getClassName(tue, args, "ampGainDriftParameter"),
          },
          {
            value: parseFloat(tue.vrefGainDrift.toPrecision(2)),
            classList: getClassName(tue, args, "vrefGainDriftParameter"),
          },
          {
            value: tue.gainDriftTotal,
          },
        ]),
      );
    }
    table.push(
      utility.getTableRow([
        {
          value: "Total",
          classList: "table-total",
        },
        {
          value: tue.adcTotal,
          classList: "table-total",
        },
        {
          value: tue.ampTotal,
          classList: "table-total",
        },
        {
          value: tue.vrefTotal,
          classList: "table-total",
        },
        {
          value: tue.uncalibratedTue,
          classList: "table-total",
        },
      ]),
    );
  }

  return {
    value: table,
  };
}

export function computeTueChart(args) {
  let calibrationType = args.cal_type.options[args.cal_type.value].display;
  if (!isTueCalculated) {
    tue.adcNoise =
      args.type.value === "Typical"
        ? args.typ_adc_noise.value
        : args.max_adc_noise.value;
    tue.adcNoise = get_value_in_selected_unit(
      tue.adcNoise,
      args.max_adc_noise.unit.value,
      args.units.value,
      args,
    );
    tue.ampNoise =
      args.type.value === "Typical"
        ? args.typ_amp_noise.value
        : args.max_amp_noise.value;
    tue.ampNoise = get_value_in_selected_unit(
      tue.ampNoise,
      args.max_amp_noise.unit,
      args.units.value,
      args,
    );

    tue.vrefNoise =
      args.type.value === "Typical"
        ? args.typ_vref_noise.value
        : args.max_vref_noise.value;
    tue.vrefNoise = get_value_in_selected_unit(
      tue.vrefNoise,
      args.max_vref_noise.unit.value,
      args.units.value,
      args,
    );

    tue.adcLinearity =
      args.type.value === "Typical"
        ? args.typ_adc_inl
          ? args.typ_adc_inl.value
          : 0
        : args.max_adc_inl
        ? args.max_adc_inl.value
        : 0;
    tue.adcLinearity = get_value_in_selected_unit(
      tue.adcLinearity,
      args.max_adc_inl.unit.value,
      args.units.value,
      args,
    );

    tue.ampLinearity =
      args.type.value === "Typical"
        ? args.typ_amp_inl
          ? args.typ_amp_inl.value
          : 0
        : args.max_amp_inl
        ? args.max_amp_inl.value
        : 0;
    tue.ampLinearity = get_value_in_selected_unit(
      tue.ampLinearity,
      args.max_amp_inl.unit.value,
      args.units.value,
      args,
    );

    tue.vrefLinearity = "-";

    tue.adcGainError =
      args.type.value === "Typical"
        ? args.typ_adc_ge
          ? args.typ_adc_ge.value
          : 0
        : args.max_adc_ge
        ? args.max_adc_ge.value
        : 0;
    tue.adcGainError = get_value_in_selected_unit(
      tue.adcGainError,
      args.max_adc_ge.unit.value,
      args.units.value,
      args,
    );

    tue.ampGainError =
      args.type.value === "Typical"
        ? args.typ_amp_ge
          ? args.typ_amp_ge.value
          : 0
        : args.max_amp_ge
        ? args.max_amp_ge.value
        : 0;
    tue.ampGainError = get_value_in_selected_unit(
      tue.ampGainError,
      args.max_amp_ge.unit.value,
      args.units.value,
      args,
    );

    tue.vrefGainError =
      args.type.value === "Typical"
        ? args.typ_vref_initial_accuracy
          ? args.typ_vref_initial_accuracy.value
          : 0
        : args.max_vref_initial_accuracy
        ? args.max_vref_initial_accuracy.value
        : 0;
    tue.vrefGainError = get_value_in_selected_unit(
      tue.vrefGainError,
      args.max_vref_initial_accuracy.unit.value,
      args.units.value,
      args,
    );

    tue.adcOffset =
      args.type.value === "Typical"
        ? args.typ_adc_offset
          ? args.typ_adc_offset.value
          : 0
        : args.max_adc_offset
        ? args.max_adc_offset.value
        : 0;
    tue.adcOffset = get_value_in_selected_unit(
      tue.adcOffset,
      args.max_adc_offset.unit.value,
      args.units.value,
      args,
    );

    tue.ampOffset =
      args.type.value === "Typical"
        ? args.typ_amp_offset
          ? args.typ_amp_offset.value
          : 0
        : args.max_amp_offset
        ? args.max_amp_offset.value
        : 0;
    tue.ampOffset = get_value_in_selected_unit(
      tue.ampOffset,
      args.max_amp_offset.unit.value,
      args.units.value,
      args,
    );

    tue.vrefOffset = "-";

    tue.adcGainDrift =
      args.type.value === "Typical"
        ? args.typ_adc_ge_drift
          ? args.typ_adc_ge_drift.value
          : 0
        : args.max_adc_ge_drift
        ? args.max_adc_ge_drift.value
        : 0;
    tue.adcGainDrift = get_value_in_selected_unit(
      tue.adcGainDrift,
      args.max_adc_ge_drift.unit.value,
      args.units.value,
      args,
    );

    tue.ampGainDrift =
      args.type.value === "Typical"
        ? args.typ_amp_ge_drift
          ? args.typ_amp_ge_drift.value
          : 0
        : args.max_amp_ge_drift
        ? args.max_amp_ge_drift.value
        : 0;
    tue.ampGainDrift = get_value_in_selected_unit(
      tue.ampGainDrift,
      args.max_amp_ge_drift.unit.value,
      args.units.value,
      args,
    );

    tue.vrefGainDrift =
      args.type.value === "Typical"
        ? args.typ_vref_temperature_drift
          ? args.typ_vref_temperature_drift.value
          : 0
        : args.max_vref_temperature_drift
        ? args.max_vref_temperature_drift.value
        : 0;
    tue.vrefGainDrift = get_value_in_selected_unit(
      tue.vrefGainDrift,
      args.max_vref_temperature_drift.unit.value,
      args.units.value,
      args,
    );

    tue.adcOffsetDrift =
      args.type.value === "Typical"
        ? args.typ_adc_offset_drift
          ? args.typ_adc_offset_drift.value
          : 0
        : args.max_adc_offset_drift
        ? args.max_adc_offset_drift.value
        : 0;
    tue.adcOffsetDrift = get_value_in_selected_unit(
      tue.adcOffsetDrift,
      args.max_adc_offset_drift.unit.value,
      args.units.value,
      args,
    );

    tue.ampOffsetDrift =
      args.type.value === "Typical"
        ? args.typ_amp_offset_drift
          ? args.typ_amp_offset_drift.value
          : 0
        : args.max_amp_offset_drift
        ? args.max_amp_offset_drift.value
        : 0;
    tue.ampOffsetDrift = get_value_in_selected_unit(
      tue.ampOffsetDrift,
      args.max_amp_offset_drift.unit.value,
      args.units.value,
      args,
    );

    tue.vrefOffsetDrift = "-";

    tue.adcTotal = get_adc_Total(args);
    tue.ampTotal = get_amp_Total(args);
    tue.vrefTotal = get_vref_Total(args);

    let errorSourcesValueswithTempError = [
      { property: "adcOffsetDriftParameter", value: tue.adcOffsetDrift },
      { property: "ampOffsetDriftParameter", value: tue.ampOffsetDrift },
      { property: "vrefGainDriftParameter", value: tue.vrefGainDrift },
      { property: "adcGainDriftParameter", value: tue.adcGainDrift },
      { property: "ampGainDriftParameter", value: tue.ampGainDrift },
      { property: "adcOffsetParameter", value: tue.adcOffset },
      { property: "ampOffsetParameter", value: tue.ampOffset },
      { property: "adcGainErrorParameter", value: tue.adcGainError },
      { property: "ampGainErrorParameter", value: tue.ampGainError },
      { property: "vrefGainErrorParameter", value: tue.vrefGainError },
      { property: "adcLinearityParameter", value: tue.adcLinearity },
      { property: "ampLinearityParameter", value: tue.ampLinearity },
      { property: "ampNoiseParameter", value: tue.ampNoise },
      { property: "adcNoiseParameter", value: tue.adcNoise },
      { property: "vrefNoiseParameter", value: tue.vrefNoise },
    ];
    errorSourcesValueswithTempError.sort((a, b) => b.value - a.value);
    tue.highestErrorSourceWithTempError = [
      errorSourcesValueswithTempError[0],
      errorSourcesValueswithTempError[1],
      errorSourcesValueswithTempError[2],
    ];

    let errorSourcesValues = [
      { property: "adcOffsetParameter", value: tue.adcOffset },
      { property: "ampOffsetParameter", value: tue.ampOffset },
      { property: "adcGainErrorParameter", value: tue.adcGainError },
      { property: "ampGainErrorParameter", value: tue.ampGainError },
      { property: "vrefGainErrorParameter", value: tue.vrefGainError },
      { property: "adcLinearityParameter", value: tue.adcLinearity },
      { property: "ampLinearityParameter", value: tue.ampLinearity },
      { property: "ampNoiseParameter", value: tue.ampNoise },
      { property: "adcNoiseParameter", value: tue.adcNoise },
      { property: "vrefNoiseParameter", value: tue.vrefNoise },
    ];
    errorSourcesValues.sort((a, b) => b.value - a.value);
    tue.highestErrorSource = [
      errorSourcesValues[0],
      errorSourcesValues[1],
      errorSourcesValues[2],
    ];

    tue.noiseTotal = get_noise_total(args);
    tue.offsetTotal = get_offset_total(args);
    tue.gainErrorTotal = get_gain_error_total(args);
    tue.inlTotal = get_inl_total(args);
    tue.calibratedOffsetDriftTotal = get_calibrated_offset_drift_total(args);
    tue.offsetDriftTotal = get_offset_drift_total(args);
    tue.calibratedGainDriftTotal = get_calibrated_gain_drift_total(args);
    tue.gainDriftTotal = get_gain_drift_total(args);
    tue.calibrationSourceAccuracy = get_value_in_selected_unit(
      args.cal_acc.value,
      args.cal_acc.unit,
      args.units.value,
      args,
    );
    calculate_and_update_tue_value(tue, calibrationType, args);
    isTueCalculated = true;
  } else {
    isTueCalculated = false;
  }
  let offsetDriftTotal = args.temp_error.value ? tue.offsetDriftTotal : 0;
  let gainDriftTotal = args.temp_error.value ? tue.gainDriftTotal : 0;
  let uncalibrated = [];
  uncalibrated.push(
    tue.noiseTotal,
    tue.offsetTotal,
    tue.gainErrorTotal,
    tue.inlTotal,
    offsetDriftTotal,
    gainDriftTotal,
  );
  let calibrated = [];
  let offsetTotal =
    calibrationType !== "No calibration" ? tue.noiseTotal : tue.offsetTotal;
  let gainErrorTotal =
    calibrationType === "Offset calibration" ||
    calibrationType === "No calibration"
      ? tue.gainErrorTotal
      : tue.calibrationSourceAccuracy;

  offsetDriftTotal =
    args.temp_error.value &&
    calibrationType === "Offset, gain, and temp calibration"
      ? tue.calibratedOffsetDriftTotal
      : offsetDriftTotal;
  gainDriftTotal =
    args.temp_error.value &&
    calibrationType === "Offset, gain, and temp calibration"
      ? tue.calibratedGainDriftTotal
      : gainDriftTotal;
  calibrated.push(
    tue.noiseTotal,
    offsetTotal,
    gainErrorTotal,
    tue.inlTotal,
    offsetDriftTotal,
    gainDriftTotal,
  );
  return {
    parameters: args.output_parameters.value,
    uncalibrated: uncalibrated,
    calibrated: calibrated,
    tueObject: tue,
    calibrationType: calibrationType,
    selectedUnit: args.units.value,
  };
}

// ---------- Helper Methods ---------------

function getClassName(tue, args, parameter) {
  if (
    args.temp_error.value &&
    (tue.highestErrorSourceWithTempError[0].property == parameter ||
      tue.highestErrorSourceWithTempError[1].property == parameter ||
      tue.highestErrorSourceWithTempError[2].property == parameter)
  ) {
    return "background-red";
  }
  if (
    !args.temp_error.value &&
    (tue.highestErrorSource[0].property == parameter ||
      tue.highestErrorSource[1].property == parameter ||
      tue.highestErrorSource[2].property == parameter)
  ) {
    return "background-red";
  }
  return "";
}

function get_Control_Object(controlData) {
  if (typeof controlData === "string" && controlData.includes("|")) {
    // Split the string by pipe characters (|)
    const valueArray = controlData.split("|");

    // Find the index of the element containing "(default)"
    const defaultValueIndex = valueArray.findIndex(element =>
      element.includes("(default)"),
    );

    // Map the array values to an object with indices
    const controlObject = {
      options: valueArray.reduce((acc, value, index) => {
        acc[index] = value.replace("(default)", "").trim();
        return acc;
      }, {}),
      value: defaultValueIndex !== -1 ? defaultValueIndex : 0,
    };

    return controlObject;
  } else if (typeof controlData === "object") {
    const valueString = controlData.value || "";
    const unitOptionsString = controlData.unitOptions || "";
    const defaultUnitString = controlData.defaultUnit || "";

    // Extract the typ and max values
    let [typValue, maxValue] = valueString.split(",");

    // Set default values if typValue or maxValue are missing
    typValue = typValue ? parseFloat(typValue.trim()) : 0;
    maxValue = maxValue ? parseFloat(maxValue.trim()) : 0;

    // Extract the default unit and options
    const unitOptions = unitOptionsString.split("|");
    const defaultUnitValue = defaultUnitString.trim();

    return {
      typ: typValue,
      max: maxValue,
      unit: defaultUnitValue,
      options: unitOptions,
    };
  }

  // Return an empty object if the input is not a string or object
  return {};
}

function get_adc_Total(args) {
  if (args.temp_error.value) {
    return parseFloat(
      Math.sqrt(
        Math.pow(tue.adcNoise, 2) +
          Math.pow(tue.adcOffset, 2) +
          Math.pow(tue.adcGainError, 2) +
          Math.pow(tue.adcLinearity, 2),
      ).toPrecision(2),
    );
  }
  return parseFloat(
    Math.sqrt(
      Math.pow(tue.adcNoise, 2) +
        Math.pow(tue.adcOffset, 2) +
        Math.pow(tue.adcGainError, 2) +
        Math.pow(tue.adcLinearity, 2) +
        Math.pow(tue.adcOffsetDrift, 2) +
        Math.pow(tue.adcGainDrift, 2),
    ).toPrecision(2),
  );
}
function get_amp_Total(args) {
  if (args.temp_error.value) {
    return parseFloat(
      Math.sqrt(
        Math.pow(tue.ampNoise, 2) +
          Math.pow(tue.ampOffset, 2) +
          Math.pow(tue.ampGainError, 2) +
          Math.pow(tue.ampLinearity, 2),
      ).toPrecision(2),
    );
  }
  return parseFloat(
    Math.sqrt(
      Math.pow(tue.ampNoise, 2) +
        Math.pow(tue.ampOffset, 2) +
        Math.pow(tue.ampGainError, 2) +
        Math.pow(tue.ampLinearity, 2) +
        Math.pow(tue.ampOffsetDrift, 2) +
        Math.pow(tue.ampGainDrift, 2),
    ).toPrecision(2),
  );
}
function get_vref_Total(args) {
  if (args.temp_error.value) {
    return parseFloat(
      Math.sqrt(
        Math.pow(tue.vrefNoise, 2) + Math.pow(tue.vrefGainError, 2),
      ).toPrecision(2),
    );
  }
  return parseFloat(
    Math.sqrt(
      Math.pow(tue.vrefNoise, 2) +
        Math.pow(tue.vrefGainError, 2) +
        Math.pow(tue.vrefGainDrift, 2),
    ).toPrecision(2),
  );
}

function get_noise_total(args) {
  let adc_noise =
    args.type.value === "Typical" ? args.typ_adc_noise : args.max_adc_noise;
  let amp_noise =
    args.type.value === "Typical" ? args.typ_amp_noise : args.max_amp_noise;
  let vref_noise =
    args.type.value === "Typical" ? args.typ_vref_noise : args.max_vref_noise;
  let noiseTotal = Math.sqrt(
    Math.pow(
      get_value_in_selected_unit(
        adc_noise.value,
        args.max_adc_noise.unit.value,
        args.units.value,
        args,
      ) / args.gain_ext.value,
      2,
    ) +
      Math.pow(
        get_value_in_selected_unit(
          amp_noise.value,
          args.max_amp_noise.unit,
          args.units.value,
          args,
        ),
        2,
      ) +
      Math.pow(
        get_value_in_selected_unit(
          vref_noise.value,
          args.max_vref_noise.unit.value,
          args.units.value,
          args,
        ) *
          (args.v_in.value / args.vref.value),
        2,
      ),
  );
  return parseFloat(noiseTotal.toPrecision(2));
}

function get_offset_total(args) {
  let adc_offset =
    args.type.value === "Typical" ? args.typ_adc_offset : args.max_adc_offset;
  let amp_offset =
    args.type.value === "Typical" ? args.typ_amp_offset : args.max_amp_offset;
  let offsetTotal = Math.sqrt(
    Math.pow(
      get_value_in_selected_unit(
        adc_offset.value,
        args.max_adc_offset.unit.value,
        args.units.value,
        args,
      ) / args.gain_ext.value,
      2,
    ) +
      Math.pow(
        get_value_in_selected_unit(
          amp_offset.value,
          args.max_amp_offset.unit.value,
          args.units.value,
          args,
        ),
        2,
      ),
  );
  return parseFloat(offsetTotal.toPrecision(2));
}

function get_gain_error_total(args) {
  let adc_ge =
    args.type.value === "Typical" ? args.typ_adc_ge : args.max_adc_ge;
  let amp_ge =
    args.type.value === "Typical" ? args.typ_amp_ge : args.max_amp_ge;
  let vref_acc =
    args.type.value === "Typical"
      ? args.typ_vref_initial_accuracy
      : args.max_vref_initial_accuracy;

  let gainErrorTotal = Math.sqrt(
    Math.pow(
      get_value_in_selected_unit(
        adc_ge.value,
        args.max_adc_ge.unit.value,
        args.units.value,
        args,
      ),
      2,
    ) +
      Math.pow(
        get_value_in_selected_unit(
          amp_ge.value,
          args.max_amp_ge.unit.value,
          args.units.value,
          args,
        ),
        2,
      ) +
      Math.pow(
        get_value_in_selected_unit(
          vref_acc.value,
          args.max_vref_initial_accuracy.unit.value,
          args.units.value,
          args,
        ),
        2,
      ),
  );
  return parseFloat(gainErrorTotal.toPrecision(2));
}

function get_inl_total(args) {
  let adc_inl =
    args.type.value === "Typical" ? args.typ_adc_inl : args.max_adc_inl;
  let amp_inl =
    args.type.value === "Typical" ? args.typ_amp_inl : args.max_amp_inl;
  let inlTotal = Math.sqrt(
    Math.pow(
      get_value_in_selected_unit(
        adc_inl.value,
        args.max_adc_inl.unit.value,
        args.units.value,
        args,
      ),
      2,
    ) +
      Math.pow(
        get_value_in_selected_unit(
          amp_inl.value,
          args.max_amp_inl.unit.value,
          args.units.value,
          args,
        ),
        2,
      ),
  );
  return parseFloat(inlTotal.toPrecision(2));
}

function get_offset_drift_total(args) {
  let adc_offset_drift =
    args.type.value === "Typical"
      ? args.typ_adc_offset_drift
      : args.max_adc_offset_drift;
  let amp_offset_drift =
    args.type.value === "Typical"
      ? args.typ_amp_offset_drift
      : args.max_amp_offset_drift;
  let offsetDriftTotal = Math.sqrt(
    Math.pow(
      get_value_in_selected_unit(
        adc_offset_drift.value,
        args.max_adc_offset_drift.unit.value,
        args.units.value,
        args,
      ) / args.gain_ext.value,
      2,
    ) +
      Math.pow(
        get_value_in_selected_unit(
          amp_offset_drift.value,
          args.max_amp_offset_drift.unit.value,
          args.units.value,
          args,
        ),
        2,
      ),
  );
  return parseFloat(offsetDriftTotal.toPrecision(2));
}

function get_calibrated_offset_drift_total(args) {
  let adc_offset_drift =
    args.type.value === "Typical"
      ? args.typ_adc_offset_drift
      : args.max_adc_offset_drift;
  let amp_offset_drift =
    args.type.value === "Typical"
      ? args.typ_amp_offset_drift
      : args.max_amp_offset_drift;
  let temp_comp_factor =
    args.cal_step.value / (args.sys_temp_max.value - args.sys_temp_min.value);
  let calibratedOffsetDriftTotal = Math.sqrt(
    Math.pow(
      (get_value_in_selected_unit(
        adc_offset_drift.value,
        args.max_adc_offset_drift.unit.value,
        args.units.value,
        args,
      ) *
        temp_comp_factor) /
        args.gain_ext.value,
      2,
    ) +
      Math.pow(
        get_value_in_selected_unit(
          amp_offset_drift.value,
          args.max_amp_offset_drift.unit.value,
          args.units.value,
          args,
        ) * temp_comp_factor,
        2,
      ),
  );
  return parseFloat(calibratedOffsetDriftTotal.toPrecision(2));
}

function get_gain_drift_total(args) {
  let adc_ge_drift =
    args.type.value === "Typical"
      ? args.typ_adc_ge_drift
      : args.max_adc_ge_drift;
  let amp_ge_drift =
    args.type.value === "Typical"
      ? args.typ_amp_ge_drift
      : args.max_amp_ge_drift;
  let vref_drift =
    args.type.value === "Typical"
      ? args.typ_vref_temperature_drift
      : args.max_vref_temperature_drift;
  let gainDriftTotal = Math.sqrt(
    Math.pow(
      get_value_in_selected_unit(
        adc_ge_drift.value,
        args.max_adc_ge_drift.unit.value,
        args.units.value,
        args,
      ),
      2,
    ) +
      Math.pow(
        get_value_in_selected_unit(
          amp_ge_drift.value,
          args.max_amp_ge_drift.unit.value,
          args.units.value,
          args,
        ),
        2,
      ) +
      Math.pow(
        get_value_in_selected_unit(
          vref_drift.value,
          args.max_vref_temperature_drift.unit.value,
          args.units.value,
          args,
        ),
        2,
      ),
  );
  return parseFloat(gainDriftTotal.toPrecision(2));
}

function get_calibrated_gain_drift_total(args) {
  let adc_ge_drift =
    args.type.value === "Typical"
      ? args.typ_adc_ge_drift
      : args.max_adc_ge_drift;
  let amp_ge_drift =
    args.type.value === "Typical"
      ? args.typ_amp_ge_drift
      : args.max_amp_ge_drift;
  let vref_drift =
    args.type.value === "Typical"
      ? args.typ_vref_temperature_drift
      : args.max_vref_temperature_drift;
  let temp_comp_factor =
    args.cal_step.value / (args.sys_temp_max.value - args.sys_temp_min.value);
  let calibratedGainDriftTotal = Math.sqrt(
    Math.pow(
      get_value_in_selected_unit(
        adc_ge_drift.value,
        args.max_adc_ge_drift.unit.value,
        args.units.value,
        args,
      ) * temp_comp_factor,
      2,
    ) +
      Math.pow(
        get_value_in_selected_unit(
          amp_ge_drift.value,
          args.max_amp_ge_drift.unit.value,
          args.units.value,
          args,
        ) * temp_comp_factor,
        2,
      ) +
      Math.pow(
        get_value_in_selected_unit(
          vref_drift.value,
          args.max_vref_temperature_drift.unit.value,
          args.units.value,
          args,
        ) * temp_comp_factor,
        2,
      ),
  );
  return parseFloat(calibratedGainDriftTotal.toPrecision(2));
}

function calculate_and_update_tue_value(tueObject, calibrationType, args) {
  let temperatureErrorIncluded = args.temp_error.value;
  if (temperatureErrorIncluded) {
    tueObject.uncalibratedTue = parseFloat(
      Math.sqrt(
        Math.pow(tue.noiseTotal, 2) +
          Math.pow(tue.offsetTotal, 2) +
          Math.pow(tue.gainErrorTotal, 2) +
          Math.pow(tue.inlTotal, 2) +
          Math.pow(tue.offsetDriftTotal, 2) +
          Math.pow(tue.gainDriftTotal, 2),
      ).toPrecision(2),
    );
  } else {
    tueObject.uncalibratedTue = parseFloat(
      Math.sqrt(
        Math.pow(tue.noiseTotal, 2) +
          Math.pow(tue.offsetTotal, 2) +
          Math.pow(tue.gainErrorTotal, 2) +
          Math.pow(tue.inlTotal, 2),
      ).toPrecision(2),
    );
  }
  switch (true) {
    case calibrationType === "Offset calibration" && temperatureErrorIncluded:
      tueObject.calibratedTue = parseFloat(
        Math.sqrt(
          Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.gainErrorTotal, 2) +
            Math.pow(tue.inlTotal, 2) +
            Math.pow(tue.offsetDriftTotal, 2) +
            Math.pow(tue.gainDriftTotal, 2),
        ).toPrecision(2),
      );
      break;
    case calibrationType === "Offset calibration" && !temperatureErrorIncluded:
      tueObject.calibratedTue = parseFloat(
        Math.sqrt(
          Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.gainErrorTotal, 2) +
            Math.pow(tue.inlTotal, 2),
        ).toPrecision(2),
      );
      break;
    case calibrationType === "Offset and gain calibration" &&
      temperatureErrorIncluded:
      tueObject.calibratedTue = parseFloat(
        Math.sqrt(
          Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.calibrationSourceAccuracy, 2) +
            Math.pow(tue.inlTotal, 2) +
            Math.pow(tue.offsetDriftTotal, 2) +
            Math.pow(tue.gainDriftTotal, 2),
        ).toPrecision(2),
      );
      break;
    case calibrationType === "Offset and gain calibration" &&
      !temperatureErrorIncluded:
      tueObject.calibratedTue = parseFloat(
        Math.sqrt(
          Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.calibrationSourceAccuracy, 2) +
            Math.pow(tue.inlTotal, 2),
        ).toPrecision(2),
      );
      break;
    case calibrationType === "Offset, gain, and temp calibration" &&
      temperatureErrorIncluded:
      tueObject.calibratedTue = parseFloat(
        Math.sqrt(
          Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.noiseTotal, 2) +
            Math.pow(tue.calibrationSourceAccuracy, 2) +
            Math.pow(tue.inlTotal, 2) +
            Math.pow(tue.calibratedOffsetDriftTotal, 2) +
            Math.pow(tue.calibratedGainDriftTotal, 2),
        ).toPrecision(2),
      );
      break;
    default:
      break;
  }
}

function get_value_in_microvolt(value, unit, args) {
  let valueInMicroVolt = value;
  switch (unit) {
    case "ppm":
    case "ppmRMS":
    case "ppmPP":
      valueInMicroVolt = value * args.v_in.value;
      break;
    case "mV":
    case "mVRMS":
    case "mVPP":
      valueInMicroVolt = value * 1000;
      break;
    case "nV":
    case "nVRMS":
    case "nVPP":
      valueInMicroVolt = value / 1000;
      break;
    case "LSB":
      let adcLsb =
        args.adc_fsr.value /
        Math.pow(2, args.parameterConfigData[args.selectedDevice].resolution);
      valueInMicroVolt = value * adcLsb * Math.pow(10, -6);
      break;
    case "%":
      valueInMicroVolt = value * 10000 * args.v_in.value;
      break;
    case "ppm/°C":
      valueInMicroVolt =
        value *
        (args.sys_temp_max.value - args.sys_temp_min.value) *
        args.v_in.value;
      break;
    case "mV/°C":
      valueInMicroVolt =
        value * 1000 * (args.sys_temp_max.value - args.sys_temp_min.value);
      break;
    case "µV/°C":
      valueInMicroVolt =
        value * (args.sys_temp_max.value - args.sys_temp_min.value);
      break;
    case "nV/°C":
      valueInMicroVolt =
        (value / 1000) * (args.sys_temp_max.value - args.sys_temp_min.value);
      break;
    case "%(FSR)":
      valueInMicroVolt = value * 10000 * args.adc_fsr.value;
      break;
    case "ppm(FSR)":
      valueInMicroVolt = value * args.adc_fsr.value;
      break;

    default:
      break;
  }
  return valueInMicroVolt;
}

function get_value_in_selected_unit(value, controlUnit, selectedUnit, args) {
  let valueInMicroVolt = get_value_in_microvolt(value, controlUnit, args);
  let finalValue = valueInMicroVolt;
  switch (selectedUnit) {
    case "ppm(VIN)":
      finalValue = valueInMicroVolt / args.v_in.value;
      break;
    case "ppm(FSR)":
      finalValue = valueInMicroVolt / args.adc_fsr.value;
      break;
    case "%(VIN)":
      finalValue = valueInMicroVolt / (args.v_in.value * 10000);
      break;
    case "%(FSR)":
      finalValue = valueInMicroVolt / (10000 * args.adc_fsr.value);
      break;
    default:
      break;
  }
  return finalValue;
}
